/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "common.h"

#include <cstdio>
#include <cstring>

#include <algorithm>
#include <vector>

std::vector<const char*> getCmdArgs(char** begin, char** end)
{
	std::vector<const char*> list;
	for (char** itr = begin+1; itr != end; itr++) {
		if (strncmp(*itr, "--", 2) != 0)
			list.push_back(*itr);
	}
	return list;
}

bool cmdOptionExists(char** begin, char** end, const std::string& option)
{
	return std::find(begin, end, option) != end;
}

static void help(const char *argv0)
{
	const char* helpDesc =
		"Usage: %s [args] <root paths or pkg name>\n"
		"		--help			- Display this screen\n"
		"		--system		- Create NI under System DLLs\n"
		"		--dll			- Create NI for DLL\n"
		"		--pkg			- Create NI for package\n"
		"\n"
		"Example:\n"
		"Create native image for dlls and exes under platform directories\n"
		"%s --system\n"
		"Create native image for dll\n"
		"%s --dll /usr/bin/Tizen.Runtime.dll\n"
		"Create native image under the package's bin and lib directory\n"
		"%s --pkg org.tizen.FormsGallery\n\n";
	printf(helpDesc, argv0, argv0, argv0, argv0);
}

int main(int argc, char* argv[])
{
	bool pkgMode = false;
	bool dllMode = false;

	if (cmdOptionExists(argv, argv+argc, "--help")) {
		help(argv[0]);
		return 0;
	} else if (cmdOptionExists(argv, argv+argc, "--system")) {
		createNiPlatform();
		return 0;
	} else if (cmdOptionExists(argv, argv+argc, "--dll")) {
		dllMode = true;
	} else if (cmdOptionExists(argv, argv+argc, "--pkg")) {
		pkgMode = true;
	} else {
		help(argv[0]);
		return 1;
	}

	std::vector<const char*> args = getCmdArgs(argv, argv+argc);

	if (args.size() < 1) {
		if (pkgMode)
			fprintf(stderr, "Package name is missed\n");
		else if (dllMode)
			fprintf(stderr, "DLL path is missed\n");
		help(argv[0]);
		return 1;
	}

	if (pkgMode) {
		for (const char* pkg : args) {
			if (createNiUnderPkgRoot(pkg) != 0) {
				fprintf(stderr, "Failed to get root path from [%s]\n", pkg);
				return 1;
			}
		}
	} else if (dllMode) {
		for (const char* dll : args)
			createNiSelect(dll);
	} else {
		createNiUnderDirs(args.data(), args.size());
	}

	return 0;
}
